(* ****************************************************************************
 *                  ZERO Inserter/Suppressor in filename
 *                  Copyright (C) 1999 Fumio Kawamata
 *
 *  This program arranges numeric portion of the filename.
 *  Arrange examples:
 *    Before
 *     pic1.jpg
 *     pic10.jpg
 *     pic11.jpg
 *     pic2.jpg
 *    After
 *     pic01.jpg
 *     pic02.jpg
 *     pic10.jpg
 *     pic11.jpg
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-
 *  ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 *  Public License in the directory "manual" for more details.
 ******************************************************************************
 *)

(* ****************************************************************************
 *
 * ****************************************************************************
 *)
unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  FileCtrl, StdCtrls, Spin, Menus, ComCtrls, Buttons, ExtCtrls;

type
  TformMain = class(TForm)
    bitbtnExit              : TBitBtn;
    bitbtnGetMax            : TBitBtn;
    directorylistboxTarget  : TDirectoryListBox;
    drivecomboboxTarget     : TDriveComboBox;
    filelistboxTarget       : TFileListBox;
    filtercomboboxTarget    : TFilterComboBox;
    imageTitle              : TImage;
    labelDirectory          : TLabel;
    labelDrive              : TLabel;
    labelFile               : TLabel;
    labelNoOfDigit          : TLabel;
    lebelExt                : TLabel;
    MainMenu1               : TMainMenu;
    menuDisplay             : TMenuItem;
    menuDisplayErrorLog     : TMenuItem;
    menuEdit                : TMenuItem;
    menuEditSelectAll       : TMenuItem;
    menuExecute             : TMenuItem;
    menuFile                : TMenuItem;
    menuFileExit            : TMenuItem;
    menuFileGetMaxLength    : TMenuItem;
    menuHelp                : TMenuItem;
    menuHelpInEnglish       : TMenuItem;
    menuHelpInJapanese      : TMenuItem;
    menuHelpVersion         : TMenuItem;
    Panel2                  : TPanel;
    panelBottom             : TPanel;
    progressbarList         : TProgressBar;
    spineditDigit           : TSpinEdit;
    StatusBar               : TStatusBar;
    procedure bitbtnExitClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure menuDisplayErrorLogClick(Sender: TObject);
    procedure menuEditSelectAllClick(Sender: TObject);
    procedure menuExecuteClick(Sender: TObject);
    procedure menuFileExitClick(Sender: TObject);
    procedure menuFileGetMaxLengthClick(Sender: TObject);
    procedure menuHelpInEnglishClick(Sender: TObject);
    procedure menuHelpInJapaneseClick(Sender: TObject);
    procedure menuHelpVersionClick(Sender: TObject);
  private
    procedure BrowseHelp(FileName:String);
    procedure RestoreOptionStatus;
    procedure StoreOptionStatus;
  public

  end;

var
  formMain: TformMain;

implementation

{$R *.DFM}

uses
  Verinfo,RegUtl,Registry,ShellAPI,Global, logview;

function ExtractFileNameWithoutExt(FullPath:string):string;
var
    Ans             : string;
    Ext             : string;
    FileNameWithExt : string;
    L               : word;
begin
  FileNameWithExt:=ExtractFileName(FullPath);
  Ext:=ExtractFileExt(FullPath);
  if Ext='' then Ans:=FileNameWithExt
  else if FileNameWithExt='' then Ans:=''
  else begin
    L:=Length(FileNameWithExt)-Length(Ext);
    Ans:=copy(FileNameWithExt,1,L);
  end;
  ExtractFileNameWithoutExt:=Ans;
end;



function ZeroPadding(Num,Digit:Integer):String;
var
  NumStr : String;
  Ans : String;
  L: Integer;
begin
  NumStr:=IntToStr(Num);
  if Length(NumStr)>=Digit then begin
    Ans:=NumStr;
  end
  else begin
    NumStr:=StringOfChar('0',Digit)+NumStr;
    L:=Length(NumStr);
    Ans:=copy(NumStr,L-Digit+1,Digit);
  end;
  ZeroPadding:=Ans;
end;



procedure TformMain.BrowseHelp(FileName:String);
var
  Code : Word;
  CodeStr : String;
  ExeDir : String;
  ExeStr : PChar;
begin
  ExeDir:=ExtractFileDir(ParamStr(0));
  ExeStr:=PChar(ExeDir+'\'+FileName);
  Code:=ShellExecute(Handle, 'open', ExeStr, nil, nil, SW_SHOWNORMAL);
  if Code<32 then begin
    str(Code, CodeStr);
    MessageDlg('Could not shellexecute "'+FileName+'".'+chr($0D)+
      'Return Code was '+CodeStr+' .', mtWarning, [mbOK],0);
  end;
end;



procedure TformMain.menuFileExitClick(Sender: TObject);
begin
  Close;
end;



procedure TformMain.menuHelpVersionClick(Sender: TObject);
(* ****************************************************************************
 * Shows Version Informations.
 * ****************************************************************************
 *)
const
  CR = chr($0D);
var
  Mess : String;
begin
  Mess:=VerInfo.GetVersionInfo('ProductName')+CR
       +VerInfo.GetVersionInfo('FileDescription')+CR
       +'Version '+VerInfo.GetVersionInfo('FileVersion')+CR
       +'Copyright '+VerInfo.GetVersionInfo('LegalCopyright')+CR
       +'Homepage '+VerInfo.GetVersionInfo('Homepage');
  MessageDlg(Mess,mtInformation,[mbOK],0);
end;




procedure TformMain.menuHelpInEnglishClick(Sender: TObject);
(* ****************************************************************************
 * Opens user help written in English.
 * The program uses HTML files for help.  Users need a web browser such as
 * Netscape Navigator or Internet Explorer.
 * ****************************************************************************
 *)
begin
  BrowseHelp('..\manual\renamer.html');
end;



procedure TformMain.menuHelpInJapaneseClick(Sender: TObject);
(* ****************************************************************************
 * Opens user help written in English.
 * The program uses HTML files for help.  Users need a web browser such as
 * Netscape Navigator or Internet Explorer.
 * ****************************************************************************
 *)
begin
  BrowseHelp('..\manual\renamer-j.html');
end;



procedure TformMain.bitbtnExitClick(Sender: TObject);
begin
  Close;
end;



procedure TformMain.StoreOptionStatus;
(* ****************************************************************************
 * Stores option statuses such as form position, wait time, panel layout, etc.
 * ****************************************************************************
 *)
var
    Reg         : TRegIniFile;
    SubKeyName  : String;
begin
  SubKeyName:=formMain.Name;
  Reg:=TRegIniFile.Create(BaseKeyName);
  Reg.WriteString(SubKeyName, 'NumOfDigit',spineditDigit.Text);
  Reg.Free;
end;



procedure TformMain.RestoreOptionStatus;
(* ****************************************************************************
 * Restores option statuses which are stored by "StoreOptionStatus" procedure.
 * ****************************************************************************
 *)
var
    Reg                 : TRegIniFile;
    SubKeyName          : String;
begin
  SubKeyName:=formMain.Name;
  Reg:=TRegIniFile.Create(BaseKeyName);
  spineditDigit.Text:=Reg.ReadString(SubKeyName,'NumOfDigit', '3');
  Reg.Free;
end;



procedure TformMain.FormDestroy(Sender: TObject);
(* ****************************************************************************
 * Stores option statuses when the registories have been existed.
 * Please note users can remove the registories for this program when it is
 * running.
 * ****************************************************************************
 *)
begin
  if ExistKey(BaseKeyName) then begin
    StoreFormStatus(formMain,BaseKeyName);
    StoreOptionStatus;
  end;
end;



procedure TformMain.FormCreate(Sender: TObject);
(* ****************************************************************************
 * Sets initial values.
 * ****************************************************************************
 *)
begin
  if not(ExistKey(BaseKeyName)) then Create_Key(BaseKeyName);
  if ExistKey(BaseKeyName+'\'+formMain.Name) then begin
    RestoreFormStatus(formMain,BaseKeyName);
    RestoreOptionStatus;
  end
  else begin
    Width:=600;
    Height:=480;
    Position:=poScreenCenter;
  end;
end;
procedure TformMain.menuDisplayErrorLogClick(Sender: TObject);
begin
  formLogViewer.Show;
end;



procedure DivideFileName(InFileName:String; var LetterPart,NumericPart,Extention:String);
var
  FileName : String;
  Finish : Boolean;
  LastLetterPos : Integer;
  L : Integer;
  S : String;
begin
  FileName:=ExtractFileNameWithoutExt(InFileName);
  Extention:=ExtractFileExt(InFileName);
  L:=Length(FileName);
  LastLetterPos:=L;
  Finish:=False;
  while Finish=False do begin
    S:=copy(FileName,LastLetterPos,1);
    if (S <'0') or (S > '9')then Finish:=True;
    if Finish=False then begin
      dec(LastLetterPos);
      if LastLetterPos=0 then Finish:=True;
    end;
  end;
  if LastLetterPos=0 then begin
    LetterPart:='';
    NumericPart:=FileName;
  end
  else begin
    LetterPart:=copy(FileName,1,LastLetterPos);
    if LastLetterPos=L then begin
      NumericPart:='';
    end
    else begin
      NumericPart:=copy(FileName,LastLetterPos+1,L-LastLetterPos);
    end;
  end;
end;



procedure TformMain.menuExecuteClick(Sender: TObject);
var
  Count: Integer;
  DriveDir : String;
  I : Integer;
  NumOfList : Integer;
  SelectedNumOfList : Integer;
  OriginalFileName : String;
  NewFileName : String;
  LetterPart : String;
  NumericPart : String;
  Extention : String;
  NumOfDigit : Integer;
  Number : LongInt;
  NewNumericPart : String;
  //MessageTxt : String;
begin
  NumOfDigit:=StrToInt(spineditDigit.Text);
  SelectedNumOfList:=filelistboxTarget.SelCount;
  if SelectedNumOfList=0 then begin
    MessageDlg('No selected file.',mtWarning,[mbOK],0);
    Exit;
  end;
  progressbarList.Min:=0;
  progressbarList.Max:=SelectedNumOfList;
  NumOfList:=filelistboxTarget.Items.Count;
  Count:=0;
  formLogViewer.memoLog.Clear;
  DriveDir:=directorylistboxTarget.Directory;
  for I:=0 to NumOfList-1 do begin
    if filelistboxTarget.Selected[I]=True then begin
      inc(Count);
      progressbarList.Position:=Count;
      OriginalFileName:=filelistboxTarget.Items[I];
      StatusBar.Panels[1].Text:=OriginalFileName;
      StatusBar.Update;
      DivideFileName(OriginalFileName,LetterPart,NumericPart,Extention);
      if NumericPart<>'' then begin
        Number:=StrToInt(NumericPart);
        NewNumericPart:=ZeroPadding(Number,NumOfDigit);
      end;
      NewFileName:=LetterPart+NewNumericPart+Extention;
      if OriginalFileName<>NewFileName then begin
        StatusBar.Panels[1].Text:=OriginalFileName+' -> '+NewFileName;
        StatusBar.Update;
        if RenameFile(DriveDir+'\'+OriginalFileName,DriveDir+'\'+NewFileName)=False then begin
          formLogViewer.memoLog.Lines.Add('Could not rename file. '+OriginalFileName+' -> '+NewFileName);
        end;
      end;
      StatusBar.Panels[1].Text:='';
    end;
  end;
  progressbarList.Position:=0;
  filelistboxTarget.Update;
end;



procedure TformMain.menuEditSelectAllClick(Sender: TObject);
var
  Count : Integer;
  I     : Integer;
begin
  Count:=filelistboxTarget.Items.Count;
  if Count=0 then begin
    Exit;
  end;
  for I:=0 to Count-1 do begin
    filelistboxTarget.Selected[I]:=True;
  end;
  filelistboxTarget.Refresh;
end;



procedure TformMain.menuFileGetMaxLengthClick(Sender: TObject);
var
  Count : Integer;
  Extention : String;
  I     : Integer;
  LetterPart : String;
  MaxLength : Integer;
  MaxNum : LongInt;
  Number : LongInt;
  NumericPart : String;
  OriginalFileName : String;
begin
  Count:=filelistboxTarget.Items.Count;
  if Count=0 then begin
    MessageDlg('No selected file.',mtWarning,[mbOK],0);
    Exit;
  end;
  MaxNum:=0;
  for I:=0 to Count-1 do begin
    if filelistboxTarget.Selected[I]=True then begin
      inc(Count);
      progressbarList.Position:=Count;
      OriginalFileName:=filelistboxTarget.Items[I];
      StatusBar.Panels[1].Text:=OriginalFileName;
      StatusBar.Update;
      DivideFileName(OriginalFileName,LetterPart,NumericPart,Extention);
      if NumericPart<>'' then begin
        Number:=StrToInt(NumericPart);
        if Number>MaxNum then MaxNum:=Number;
      end;
    end;
  end;
  StatusBar.Panels[1].Text:='';
  progressbarList.Position:=0;
  MaxLength:=Length(IntToStr(MaxNum));
  spineditDigit.Value:=MaxLength;
  spineditDigit.Update;
end;

end.
